// Copyright (c) 2014, Vuzix Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// *  Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// *  Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// *  Neither the name of Vuzix Corporation nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
package com.vuzix.shapepusher;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.LinearLayout;

import com.vuzix.hardware.GestureSensor;

public class ShapePusherActivity extends Activity {

    private MyGestureSensor mGS;
    private Canvas mCanvas;
    private Bitmap mBG;

    private int mX;
    private int mY;
    private int mSize;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_shape_pusher);

        // Set up Gesture Sensor
        mGS = new MyGestureSensor(this);

        if (mGS == null) {
            android.util.Log.i("VUZIX-Gesture","Unable to create MyGestureSensor()");
            return;
        }

        android.util.Log.i("VUZIX-Gesture", "registering MyGestureSensor");
        mGS.register();

        // Set default values for circle
        mX = 215;
        mY = 100;
        mSize = 40;

        // Set up drawing canvas
        mBG = Bitmap.createBitmap(432,200,Bitmap.Config.ARGB_8888);
        mCanvas = new Canvas(mBG);

        // Draw initial circle
        drawShape(Color.BLUE);

    }

    @Override
    public void onDestroy()
    {
        // If we have an active Gesture Recognizer object, call
        // unregister() just to be safe (stop receiving gesture events)
        if (mGS != null)
        {
            mGS.unregister();
        }

        super.onDestroy();
    }

    @Override
    public void onPause()
    {
        // If we have an active Gesture Recognizer object, call
        // unregister() just to be safe (stop receiving gesture events)
        if (mGS != null)
        {
            mGS.unregister();
        }

        super.onPause();
    }

    @Override
    public void onResume()
    {
        super.onResume();

        // If we have an active Gesture Recognizer object, call
        // register() to begin receiving gesture events again
        if (mGS != null)
        {
            mGS.register();
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_shape_pusher, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    // draw a shape.   Takes an instance of ShapeInfo and a color.
    // We could pull the color from the ShapeInfo structure except in the
    // case where we are calling drawShape() to ERASE the shape.
    private void drawShape(int color)
    {
        Paint paint = new Paint();
        paint.setColor(color);

        mCanvas.drawCircle(mX, mY, mSize, paint);

        LinearLayout linearLayout = (LinearLayout) findViewById(R.id.rect);
        linearLayout.setBackgroundDrawable(new BitmapDrawable(mBG));
    }

    // Here is the nested Gesture Sensor class used to retrieve
    // events from the Gesture Sensor
    public class MyGestureSensor extends GestureSensor {
        public MyGestureSensor(android.content.Context mContext) {
            super(mContext);
        }

        // Gesture Callbacks
        @Override
        protected void onBackSwipe(int speed) {
            android.util.Log.i("VUZIX-Gesture", "onBackSwipe(), speed=" + speed);
            if (mSize < 100) {
                drawShape(Color.WHITE);
                mSize += 20;
                drawShape(Color.BLUE);
            }
        }

        @Override
        protected void onForwardSwipe(int speed) {
            android.util.Log.i("VUZIX-Gesture","onForwardSwipe(), speed="+speed);

            if (mSize > 20) {
                drawShape(Color.WHITE);
                mSize -= 20;
                drawShape(Color.BLUE);
            }
        }

        @Override
        protected void onNear() {
            android.util.Log.i("VUZIX-Gesture", "onNear()");
            if (mX > 20) {
                drawShape(Color.WHITE);
                mX -= 20;
                drawShape(Color.BLUE);
            }
        }

        @Override
        protected void onFar() {
            android.util.Log.i("VUZIX-Gesture", "onFar()");
            if (mX < 400) {
                drawShape(Color.WHITE);
                mX += 20;
                drawShape(Color.BLUE);
            }

        }

        @Override
        protected void onDown(int speed) {
            // on Down() must be overridden, but the gesture is not enabled
            android.util.Log.i("VUZIX-Gesture","onDown(), speed="+speed);
        }

        @Override
        protected void onUp(int speed) {
            // onUp() must be overridden, but the gesture is not enabled.
            android.util.Log.i("VUZIX-Gesture","onUp(), speed="+speed);
        }
    }
}
